/************************************************************************

    Quick And Dirty Apricot Emulator

    Copyright (C) 2005-6  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include <SDL.h>
#include <string>
#include "qdae.hxx"
#ifdef HAVE_WINDOWS_H
#include <windows.h>
#endif
#include "UiControl.hxx"
#include "UiLabel.hxx"
#include "UiCommand.hxx"
#include "UiFileEntry.hxx"

extern string displayName (string filename, int len);


UiFileEntry::UiFileEntry(string filename, string dirname, UiDrawer *d)
		: UiCommand(SDLK_UNKNOWN, string("  ")+filename+string("  "), d)
{
	m_filename = filename;
	m_dirname  = dirname;
	m_statted = false;
}

UiFileEntry::~UiFileEntry()
{

}

void UiFileEntry::draw(int selected)
{
	stat();
	m_caption = "  " + displayName(m_filename, 36) + "  ";
	UiCommand::draw(selected);
}

#ifdef HAVE_WINDOWS_H
WIN32_FIND_DATA &UiFileEntry::stat(void)
{
	if (!m_statted)
	{
		// Special case: These two are always directories
		if (m_filename == "." || m_filename == "..")
		{
			m_statted = true;
			m_st.dwFileAttributes = FILE_ATTRIBUTE_DIRECTORY;
			setGlyph(UIG_SUBMENU);
		}
		HANDLE hFind = ::FindFirstFile((m_dirname + m_filename).c_str(), &m_st);

		if (hFind != INVALID_HANDLE_VALUE)
		{	
			m_statted = true;
			if (m_st.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) setGlyph(UIG_SUBMENU);
			::FindClose(hFind);
		}
	}
	return m_st;
}
#else


struct stat &UiFileEntry::stat(void)
{
	if (!m_statted)
	{
		if (!::stat((m_dirname + m_filename).c_str(), &m_st)) 
		{	
			m_statted = true;
			if (S_ISDIR(m_st.st_mode)) setGlyph(UIG_SUBMENU);
		}
	}
	return m_st;
}

#endif
