/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2007  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/


class PC_ENV;
class I8089;

class I8089Channel
{
	friend class I8089;

	PC_ENV *m_host;
	I8089 *m_parent;
	bool m_running;
	int m_xferPending;
	int m_countdown;
	int m_offset;
	int m_sourcew, m_destw;
	int m_mmcopy;
	u_int8 m_ccw;
	u_int16 m_pboff, m_pbseg;
	u_int16 m_tboff, m_tbseg;
/* The registers */
	int 	m_ga, m_gb, m_gc, m_tp, m_pp, m_ip;	/* 20-bit registers */
	u_int16 m_bc, m_ix, m_cc, m_mc;	/* 16-bit registers */

	void *m_regptr[8];
	int  *m_baseptr[4];

	int addr();
	void reset();
	void tick();
	void execute();
	void xfer();
	int getBase(int *baseptr, int mode);
	int fetchImmediate(int wb);
	void setReg(int reg, int value);
	void adjustReg(int reg, int value);
// Store and fetch operations
	u_int8  fetch8(int base, int addr);
	u_int16 fetch16(int base, int addr);
	void store8(int base, int addr, u_int8 value);
	void store16(int base, int addr, u_int16 value);
// Disassembly helpers
	const char *rrrName(int reg);
	const char *pppName(int reg);
	const char *basePtrName(int reg);
	const char *immediateName(int ih);
	const char *memoryName(int ih, int il);
};

class I8089 : public Device
{
	friend class I8089Channel;

	bool m_inited;
	u_int8 m_buswidth, m_soc;
	u_int16 m_cboff, m_cbseg;

	I8089Channel m_ch[2];

	void initialise(void);

public:
	I8089(PC_ENV *host);
	~I8089();

	void write(u_int16 port, u_int16 value);
	u_int16 read(u_int16 port);
	void tick(void);

	virtual void reset();
	virtual bool parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);
	virtual bool storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);
	virtual UiEvent settings(UiDrawer *d);
	virtual bool hasSettings(SDLKey *key, string &caption);
private:
	PC_ENV *m_host;
};

