/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2007  John Elliott <jce@seasip.demon.co.uk>

*************************************************************************/

// Based largely on e8253.c/h from pce by Hampa Hug. //
//////////////////////////////////////////////////////////////////////////////
// This program is free software. You can redistribute it and / or modify it /
// under the terms of the GNU General Public License version 2 as  published //
// by the Free Software Foundation.                                          //
//                                                                           //
// This program is distributed in the hope  that  it  will  be  useful,  but //
// WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General //
// Public License for more details.                                          //
///////////////////////////////////////////////////////////////////////////////


#include "qdae.hxx"
#include "device.hxx"
#include "i8253.hxx"

//#define TMR_LOG(x) printf x ; fflush(stdout)
#define TMR_LOG(x)

TimerChannel::TimerChannel(I8253 *par, int ch)
{
	m_out = NULL;
	m_parent = par;
	m_channel = ch;
	init();
}


void TimerChannel::init()
{
	m_counter = 0;
	m_counterWrite = 0;
	m_outputLatch = 0;
	m_outputLatchRead = 0;
	m_counterRead = 0;
	m_rw = 3;
	m_mode = 0;
	m_bcd = false;
	m_counting = false;
	m_gate = false;
	m_outVal = false;
	m_val = 0;
}


void TimerChannel::setOut(bool val)
{
	if (m_channel == 0) TMR_LOG(("Timer(%d): setOut(%d)\n", m_channel, val));
	if (m_outVal != val)
	{
		m_outVal = val;
		if (m_out != NULL)
		{
			if (m_channel == 0) TMR_LOG(("Timer(%d): Fire (%d)\n", m_channel, val));
			m_out->onTimerFire(m_parent, m_channel, m_outVal);
		}
	}
}


void TimerChannel::newVal0()
{
	setOut(false);
	m_val = m_counter;
	
	TMR_LOG(("Timer(%d): New value %d\n", m_channel, m_val));
	if (m_gate) m_counting = true;
}
	
void TimerChannel::decVal0(int n)
{
	while (n > 0)
	{
		if (m_gate && !m_outVal)
		{
			m_val = (short)((m_val - 1) & 0xFFFF);
			if (m_val == 0)
			{
				setOut(true);
			}
		}
		--n;
	}
}
	
void TimerChannel::setGate0(bool val)
{
	m_counting = (m_counting && val);
}
	
void TimerChannel::newVal2()
{
	setOut(true);
	m_val = m_counter;
	if (m_gate) m_counting = true;
}

void TimerChannel::decVal2(int n)
{
	while (n > 0)
	{
		if (n < m_val)
		{
			m_val -= n;
			n = 0;
		}
		else if (m_val > 1)
		{
			n -= (m_val - 1);
			m_val = 1;
		}
		else 
		{
			m_val = (short)((m_val - 1) & 0xFFFF);
			--n;
		}
	}
	if (m_val == 1) setOut(false);
	else if (m_val == 0) newVal2();
}
	
void TimerChannel::setGate2(bool val)
{
	if (val)
	{
		if (m_gate == false && m_counterWrite == 0)
		{
			newVal2();
			m_counting = true;
		}
	}
	else m_counting = false;
}

void TimerChannel::newVal3()
{
	if (!m_counting)
	{
		setOut(true);
		m_val = m_counter;
		m_val &= 0xFFFE;
		if (m_gate) m_counting = true;
	}
}
	
void TimerChannel::decVal3(int n)
{
	while (n > 0)
	{
		if (m_val == 0)
		{
			m_val = m_counter;
			m_val = (short)((m_val - 2) & 0xfffe);
			--n;
		}
		else if ( (n << 1) <= (0xFFFF & m_val))
		{
			m_val -= (n << 1);
			n = 0;
		}
		else
		{
			n -= (m_val >> 1);
			m_val = 0;
		}
	}
	if (m_val == 0) setOut(!m_outVal);
}
	
void TimerChannel::setGate3(bool val)
{
	if (val)
	{
		if (m_gate == false && m_counterWrite == 0)
		{
			newVal3();
			m_counting = true;
		}
	}
	else
	{
		setOut(true);
		m_counting = false;
	}
}

void TimerChannel::newVal4()
{
	setOut(true);
	m_val = m_counter;	
	if (m_gate) m_counting = true;
}

void TimerChannel::decVal4(int n)
{
	int pval = (m_val & 0xFFFF);
	while (n > 0)
	{
	/* Reduction would not take it near 0 */
		if (n < pval)
		{
			m_val -= n;
			n = 0;
		}
	/* Reduction would take it to 0 */
		else if (pval > 1)
		{
			n -= (pval - 1);
			m_val = 1;
		}
	/* n >= pval && pval <= 0 */
		else 
		{
			m_val = (short)((pval - 1) & 0xFFFF);
			--n;
		}
		if (m_val == 1) setOut(false);
		else if (m_val == 0) setOut(true);
	}
}
		
void TimerChannel::setGate4(bool val)
{
	m_counting = (m_counting && val);
}

void TimerChannel::newVal()
{
	switch(m_mode)
	{
		case 0: newVal0(); break;
		case 2: 
		case 6: newVal2(); break;
		case 3: 
		case 7: newVal3(); break;
		case 4: newVal4(); break;
	}
}
	
void TimerChannel::decVal(int n)
{
	switch(m_mode)
	{
		case 0: decVal0(n); break; 
		case 2: 
		case 6: decVal2(n); break; 
		case 3: 
		case 7: decVal3(n); break; 
		case 4: decVal4(n); break;
	}
}
		
u_int8 TimerChannel::getValue()
{
	if ((m_outputLatchRead & 1) != 0)
	{
		m_outputLatchRead &= ~1;
		return (u_int8)(m_outputLatch & 0xFF);
	}
	if ((m_outputLatchRead & 2) != 0)
	{
		m_outputLatchRead &= ~2;
		return (u_int8)((m_outputLatch >> 8) & 0xFF);
	}	
	if (m_counterRead == 0)
	{
		m_counterRead = m_rw;
	}
	if ((m_counterRead & 1) != 0)
	{
		m_counterRead &= ~1;
		return (u_int8)(m_val & 0xFF);
	}
	m_counterRead &= ~2;
	return (u_int8)((m_val >> 8) & 0xFF);
}
	
void TimerChannel::setValue(u_int8 val)
{
	TMR_LOG(("Timer(%d): setValue %d\n", m_channel, val));
	if (m_counterWrite == 0)
	{
		m_counterWrite = m_rw;
	}
	if ((m_counterWrite & 1) != 0)
	{
		m_counterWrite &= ~1;
		m_counter &= 0xFF00;
		m_counter |= (0xFF & val);
/* Weird Apricot thing: It sets up a 16-bit mode. Then it writes 47h and
 * immediately rereads, and if it didn't get ~47h it fails the timer */
		if (m_counterWrite != 0)
		{
			m_val = ~m_counter;
		}
	}
	else if ((m_counterWrite & 2) != 0)
	{
		m_counterWrite &= ~2;
		m_counter &= 0xFF;
		m_counter |= (0xFF00 & (((int)val) << 8));
	}
	if (m_counterWrite == 0)
	{
		newVal();
	}
}
	
void TimerChannel::setGate(bool value)
{
	if (value != m_gate)
	{
		switch(m_mode)
		{
			case 0: setGate0(value); break;
			case 6:
			case 2: setGate2(value); break;
			case 7:
			case 3: setGate3(value); break;
			case 4: setGate4(value); break;
			
		}
		m_gate = value;
	}
}
	
void TimerChannel::latch()
{
	if (m_outputLatchRead != 0) return;
	
	m_outputLatchRead = m_rw;
	m_outputLatch = m_val;
}
	
void TimerChannel::setControl(u_int8 val)
{
	if ((val & 0xC0) == 0xC0)
	{
		// 8254 read back, not supported
		return;
	}
	u_int8 rw = (u_int8)((val >> 4) & 3);
	TMR_LOG(("Timer(%d): setControl rw=%d\n", m_channel, rw));
	if (rw == 0)
	{
		latch();
		return;
	}
	m_sr = val;
	m_counterWrite = rw;
	m_outputLatchRead = 0;
	m_counterRead = rw;
	m_counting = false;
	m_rw = rw;
	m_mode = (u_int8)((val >> 1) & 7);
	m_bcd = (val & 1) != 0;
	TMR_LOG(("Timer(%d): rw=%d mode=%d bcd=%d\n", 
				m_channel, rw, m_mode, m_bcd));
}





I8253::I8253(PC_ENV *host)
{
	floatSteps = 0;
	stepSpeed = 8.0 / 4.77;	// 8 divided by CPU speed
	m_host = host;
	for (int n = 0; n < I8253_CHANNELS; n++)
	{
		m_channel[n] = new TimerChannel(this, n);
	}
	reset();
}


I8253::~I8253()
{
	for (int n = 0; n < I8253_CHANNELS; n++)
	{
		delete m_channel[n];
	}
}


void I8253::write(u_int16 port, u_int8 value)
{
	int ch;

	TMR_LOG(("Timer: Out %d, %02x\n", port, value));
	switch(port)
	{
		case 0: 
		case 1:
		case 2:	m_channel[port]->setValue(value);
			break;
		case 3:	ch = ((value >> 6) & 3);
			if (ch < I8253_CHANNELS)
			{
				m_channel[ch]->setControl(value);
			}
			break;
	}
}


u_int8 I8253::read(u_int16 port)
{
	u_int8 value = 0xFF;
	if (port < I8253_CHANNELS)
	{
		value = m_channel[port]->getValue();
	}
	TMR_LOG(("Timer: Read %d = %02x\n", port, value));
	return value;
}

void I8253::reset()
{
	for (int n = 0; n < I8253_CHANNELS; n++)
	{
		m_channel[n]->init();
	}
	stepSpeed = 1.0;
}


bool I8253::parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	return true;
}


bool I8253::storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	return true;
}

void I8253::tick()
{
/* QDAE bodge: hardcode the amount the timers get decremented at each 
 * instruction fetch, so that it passes self-test.
 *
 * Roughly, nos. 1 and 2 go at 2MHz and 0 goes at 0.25MHz, so get no. 2
 * right enough to pass POST and set up number 0 accordingly.
 * */
/*
	floatSteps += stepSpeed;
	int step = (int)floatSteps;
	floatSteps -= (float)step;
	if (step == 0) return;
*/
	static int step[I8253_CHANNELS] = { 0x03, 0x18, 0x18 };

	for (int n = 0; n < I8253_CHANNELS; n++)
	{
		if (m_channel[n]->isCounting())
		{
		//	m_channel[n]->decVal(step);
			m_channel[n]->decVal(step[n]);
		}
	}
}

void I8253::setListener(int channel, TimerListener *tl)
{
	if (channel >= 0 && channel < I8253_CHANNELS)
		m_channel[channel]->setTimerListener(tl);
}

void I8253::setGate(int channel, bool v)
{
	if (channel >= 0 && channel < I8253_CHANNELS)
		m_channel[channel]->setGate(v);	
}
