/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include "qdae.hxx"
#include "qdae_files.hxx"
#include "qdae_ui.hxx"
#include "sdl_video.hxx"
#include "f1_video.hxx"
#include "rascal.hxx"
#include "machine_f1.hxx"

#include "sdl_keyboard.hxx"
#include "f1_keyboard.hxx"
#include "memsize.hxx"

#define XMLNS             "http://www.seasip.demon.co.uk/qdae_f1.dtd"
#define HW_FILE           "fseries.xml" 

F1_ENV::F1_ENV() :
	Rascal_ENV(HW_FILE, XMLNS),
	m_ctc(this),
	m_sio(this),
	m_fdc(this),
	m_winch(this)
{
	m_keyboard = NULL;
	m_video = NULL;
}

F1_ENV::~F1_ENV()
{
	if (m_keyboard) delete m_keyboard;
	if (m_video) delete m_video;
}

PC_ENV *F1_ENV::clone()
{
	F1_ENV *other = new F1_ENV();
	copy_to(other);
	other->m_ctc = m_ctc;
	other->m_fdc = m_fdc;
	other->m_winch = m_winch;
	return other;
}

void F1_ENV::initialize()
{
	if (!m_keyboard) m_keyboard = new F1_Keyboard(this);
	if (!m_video)    m_video    = new F1_Video(this);

	addDevice(&m_ctc);
	addDevice(&m_sio);
	addDevice(&m_fdc);
	addDevice(&m_winch);
	addDevice(m_keyboard);
	addDevice(m_video);
	loadHardware();
	parseHardware();

	PC_ENV::initialize();

	load_rom("f1.rom", 0xF8000, 0x8000);
}



void F1_ENV::sys_out8(u_int16 port, u_int8 value)
{
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		m_winch.write(port - 0x1E0, value);
		return;
	}
	switch(port)
	{
		case 0x30:
			m_ctc.opcode(value);
			m_sio.opcode(value);
			break;
		case 0x20:
		case 0x22:
		case 0x24:
		case 0x26:
			m_sio.write((port & 6) / 2, value);
			break;
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
			//m->debug |= DEBUG_DECODE_F | DEBUG_TRACE_F;
			m_ctc.write((port & 6) / 2, value);
			break;
		case 0x01:
			m_fdc.select_drive((~value) & 1);
			break;
		case 0x03:
			m_fdc.load_head(value != 0);
			break;
		case 0x05:
			m_fdc.set_motor(value);
			break;
		case 0x40:
		case 0x42:
		case 0x44:
		case 0x46:
			m_fdc.write((port & 6) / 2, value);
			break;
		case 0x07:
		case 0x09:
		case 0x0B:
		case 0x0D:
			m_video->write(port,value);
			break;
		case 0x00:
		case 0x0F:
/* XXX Implement! 	printer_write(port, value); */
			break;	
		default:
			fprintf(stderr, "Unknown OUT at %04x:%04x: "
					"port=%04x value=%02x\n", 
					R_CS, R_IP, port, value); 
			break;
	}
}


void F1_ENV::sys_out16(u_int16 port, u_int16 value)
{
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		m_winch.write(port, value & 0xFF);
		return;
	}
	switch(port)
	{

		case 0x30:
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
		case 0x20:
		case 0x22:
		case 0x24:
		case 0x26:
		case 0x01:
		case 0x05:
		case 0x40:
		case 0x42:
		case 0x44:
		case 0x46:
			sys_out8(port, value & 0xFF);
			break;
		default:
/*			fprintf(stderr, "Unknown OUT at %04x:%04x: "
					"port=%04x value=%04x\n", 
					m->R_CS, m->R_IP, port, value); */
			break;
	}
}

u_int8 F1_ENV::sys_in8(u_int16 port)
{
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		return m_winch.read(port - 0x1E0);
	}
	switch(port)
	{
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
			return m_ctc.read((port & 6) / 2);
		case 0x20:
		case 0x22:
		case 0x24:
		case 0x26:
			return m_sio.read((port & 6) / 2, false);
		case 0x40:
		case 0x42:
		case 0x44:
		case 0x46:
			return m_fdc.read((port & 6) / 2);
			break;
				
		default:
/*			fprintf(stderr, "Unknown IN8: port=%04x\n", port); */
			break;
	}
	return 0xFF;
}


u_int16 F1_ENV::sys_in16(u_int16 port)
{
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		return m_winch.read(port - 0x1E0);
	}
	switch(port)
	{
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
		case 0x20:
		case 0x22:
		case 0x24:
		case 0x26:
		case 0x40:
		case 0x42:
		case 0x44:
		case 0x46:
			return sys_in8(port);
		default:
/*			fprintf(stderr, "Unknown IN16: port=%04x\n", port); */
			break;
	}
	return 0xFFFF;
}


void F1_ENV::clock_tick()
{
	PC_ENV::clock_tick();

	m_ctc.tick();
	m_sio.tick();
	/* XXX Speed regulation */
}



void F1_ENV::reboot()
{
	m_ctc.reset();
	m_sio.reset();
	m_fdc.reset();
	m_winch.reset();
	/* XXX Add resets for other peripherals */
	PC_ENV::reboot();
}


UiEvent F1_ENV::floppy_insert(int drive, std::string filename, const char *type)
{
	m_fdc.insert(drive, filename, type);
}

void F1_ENV::floppy_eject(int drive)
{
	m_fdc.eject(drive);
}


void F1_ENV::write_infrared(u_int16 value)
{
	m_sio.write_ir(0, value);
}

void F1_ENV::poll_events()
{
	if (gl_f10count)
	{
		--gl_f10count;
		if (!gl_f10count) m_keyboard->send_time();
	}
	PC_ENV::poll_events();
}


u_int8 F1_ENV::fetch(u_int32 offset)
{
	if (offset >= m_memsize->m_memsize && offset < 0xE0000) return 0xFF;
	return m_mem_base[offset];
}

void F1_ENV::store(u_int32 offset, u_int8 value)
{
	if (offset < m_memsize->m_memsize || 
	(offset >= 0xE0000 && offset < 0xF8000))  m_mem_base[offset] = value;
}


void F1_ENV::sys_lock()
{
}

