/* 
 * Dos/PC Emulator
 * Copyright (C) 1991 Jim Hudgens
 * Copyright 2005,2007 John Elliott 
 * 
 * The file is part of GDE.
 * 
 * GDE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 * 
 * GDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GDE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 *
 */

#include "device.hxx"
#include "z80_sio.hxx"
#include "wd2797.hxx"
#include "i8259.hxx"
#include "i8253.hxx"
#include "winchester.hxx"

class FP_ENV : public Rascal_ENV
{
protected:
	u_int16	m_ramSelect;
public:
	FP_ENV();
	virtual ~FP_ENV();
	virtual void initialize();
	virtual void reboot();

	/* Callbacks from CPU emulation */
	virtual void clock_tick();
	virtual void sys_out8(u_int16 port, u_int8 value);
	virtual void sys_out16(u_int16 port, u_int16 value);
	virtual u_int16 sys_in16(u_int16 port);
	virtual u_int8 sys_in8(u_int16 port);
	virtual void sys_lock(void);

	/* Kludge for the debugger */
	virtual PC_ENV *clone();

	/* Floppy drive public methods */
        virtual void floppy_eject(int drive);
        virtual UiEvent floppy_insert(int drive, std::string filename,
                        const char *type);

	/* Input from infrared keyboard comes here */
	virtual void write_infrared(u_int16 value);

	virtual void poll_events();

	virtual u_int8 fetch(u_int32 offset);
	virtual void   store(u_int32 offset, u_int8 value);

protected:
	void video_init(void);
	void keyboard_init(void);

private:
	Z80_SIO m_sio;
	WD2797 m_fdc;
	I8259 m_pic;
	I8253 m_pit;
	Winchester m_winch;
};

