/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005,2007  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include "qdae.hxx"
#include "qdae_files.hxx"
#include "qdae_ui.hxx"
#include "sdl_video.hxx"
#include "xi_video.hxx"
#include "machine_xi.hxx"

#include "sdl_keyboard.hxx"
#include "xi_keyboard.hxx"
#include "memsize.hxx"

#define XMLNS             "http://www.seasip.demon.co.uk/qdae_xi.dtd"
#define HW_FILE           "pcxi.xml" 


XI_ENV::XI_ENV() :
	PC_ENV(HW_FILE, XMLNS),
	m_pio(this),
	m_pic(this),
	m_sio(this),
	m_fdc(this),
	m_pit(this),
	m_8089(this),
	m_winch(this)
{
	m_keyboard = NULL;
	m_video = NULL;
	m_memsize->m_max = 0xF0000;
}

XI_ENV::~XI_ENV()
{
	if (m_keyboard) delete m_keyboard;
	if (m_video) delete m_video;
}

PC_ENV *XI_ENV::clone()
{
	XI_ENV *other = new XI_ENV();
	copy_to(other);
	other->m_pio = m_pio;
	other->m_pic = m_pic;
	other->m_pit = m_pit;
	other->m_sio = m_sio;
	other->m_8089 = m_8089;
	other->m_fdc = m_fdc;
	other->m_winch = m_winch;
	return other;
}

void XI_ENV::initialize()
{
	XI_Keyboard *kbd = (XI_Keyboard *)m_keyboard;
	XI_Video    *vid = (XI_Video    *)m_video;

	if (!kbd)	m_keyboard = kbd = new XI_Keyboard(this);
	if (!vid)	m_video    = vid = new XI_Video   (this);

	addDevice(&m_sio);
	addDevice(&m_pio);
	addDevice(&m_fdc);
	addDevice(&m_pic);
	addDevice(&m_pit);
	addDevice(&m_8089);
	addDevice(&m_winch);
	addDevice(m_keyboard);
	addDevice(m_video);

	PC_ENV::initialize();

	m_sio.setListener(1, ((XI_Keyboard *)m_keyboard));
	/* Put the SIO on IRQ5 */
	m_sio.setPIC(&m_pic, 5);

	/* Hook the keyboard up to the MicroScreen */
	kbd->setMicroscreen(vid->getMicroscreen());
	addDevice(vid->getMicroscreen());
	loadHardware();
	parseHardware();
	load_rom("pc.rom", 0xFC000, 0x4000, "xi.rom");

/* In the PC/XI, the floppy motors are always on */
	m_fdc.set_motor(1);
}



void XI_ENV::sys_out8(u_int16 port, u_int8 value)
{
/*
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		m_winch.write(port - 0x1E0, value);
		return;
	} */
	switch(port)
	{
		case 99: settings.debug = value; break;

		case 0x00:
		case 0x02:
			m_pic.write(port / 2, value);
			break;
		case 0x40:
		case 0x42:
		case 0x44:
		case 0x46:
			m_fdc.write((port & 6) / 2, value);
			break;
		case 0x48:
		case 0x4A:
		case 0x4C:
		case 0x4E:
			m_pio.write((port & 6) / 2, value);
			break;
		case 0x50:
			/* Sound generator, not implemented */
			break;
		case 0x58:
		case 0x5A:
		case 0x5C:
		case 0x5E:
			m_pit.write((port & 6) / 2, value);
			break;
		case 0x60:
		case 0x62:
		case 0x64:
		case 0x66:
			m_sio.write((port & 6) / 2, value);
			break;
		case 0x68:
		case 0x6A:
			m_video->write((port & 2) / 2,value);
			break;
		case 0x70:
		case 0x72:
			m_8089.write((port & 2) / 2, value);
			break;
		/*
		case 0x30:
			m_ctc.opcode(value);
			m_sio.opcode(value);
			break;
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
			//m->debug |= DEBUG_DECODE_F | DEBUG_TRACE_F;
			m_ctc.write((port & 6) / 2, value);
			break;
		case 0x01:
			m_fdc.select_drive((~value) & 1);
			break;
		case 0x05:
			m_fdc.set_motor(value);
			break;
		case 0x07:
		case 0x09:
		case 0x0B:
		case 0x0D:
			m_video->write(port,value);
			break;
		case 0x00:
		case 0x0F:
// XXX Implement! 	printer_write(port, value);
			break;	
*/
		default:
			fprintf(stderr, "Unknown OUT at %04x:%04x: "
					"port=%04x value=%02x\n", 
					R_CS, R_IP, port, value); 
			break;
	}
}


void XI_ENV::sys_out16(u_int16 port, u_int16 value)
{
	/*
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		m_winch.write(port, value & 0xFF);
		return;
	} */
	switch(port)
	{
		case 99: settings.debug = value; break;
		case 0x70:
		case 0x72:
			m_8089.write((port & 2) / 2, value);
			break;
		case 0x00:
		case 0x02:
		case 0x48:
		case 0x4A:
		case 0x4C:
		case 0x4E:
		case 0x40:
		case 0x42:
		case 0x44:
		case 0x46: 
		case 0x58:
		case 0x5A:
		case 0x5C:
		case 0x5E:
		case 0x60:
		case 0x62:
		case 0x64:
		case 0x66:
		case 0x68:
		case 0x6A:
/*
		case 0x30:
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
		case 0x20:
		case 0x22:
		case 0x24:
		case 0x26:
		case 0x01:
		case 0x05:
			*/
			sys_out8(port, value & 0xFF);
			break;
		default:
			fprintf(stderr, "Unknown OUT at %04x:%04x: "
					"port=%04x value=%04x\n", 
					R_CS, R_IP, port, value); 
			break;
	}
}

u_int8 XI_ENV::sys_in8(u_int16 port)
{
/*
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		return m_winch.read(port - 0x1E0);
	} */
	switch(port)
	{ 
		case 99: return settings.debug; 
		case 0x00:
		case 0x02:
			return m_pic.read(port / 2);
		case 0x40:
		case 0x42:
		case 0x44:
		case 0x46:
			return m_fdc.read((port & 6) / 2);
			break;
		case 0x48:
		case 0x4A:
		case 0x4C:
		case 0x4E:	
			return m_pio.read((port & 6) / 2);
			break;
		case 0x58:
		case 0x5A:
		case 0x5C:
		case 0x5E:
			return m_pit.read((port & 6) / 2);
		case 0x60:
		case 0x62:
		case 0x64:
		case 0x66:
			return m_sio.read((port & 6) / 2, (port == 0x60 && m_locked));
		case 0x68:
		case 0x6A:
			return m_video->read((port & 2) / 2);
		case 0x70:
		case 0x72:
			return m_8089.read((port & 2) / 2) & 0xFF;
		/*
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
			return m_ctc.read((port & 6) / 2);
*/		
		default:
			fprintf(stderr, "Unknown IN8: port=%04x\n", port); 
			break;
	}
	m_locked = false;
	return 0xFF;
}


u_int16 XI_ENV::sys_in16(u_int16 port)
{
/*
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		return m_winch.read(port - 0x1E0);
	} */
	switch(port)
	{
		case 99: return settings.debug; 
		case 0x70:
		case 0x72:
			return m_8089.read((port & 2) / 2);
		case 0x00:
		case 0x02:
		case 0x40:
		case 0x42:
		case 0x44:
		case 0x46:
		case 0x48:
		case 0x4A:
		case 0x4C:
		case 0x4E:	
		case 0x50:
		case 0x58:
		case 0x5A:
		case 0x5C:
		case 0x5E:
		case 0x60:
		case 0x62:
		case 0x64:
		case 0x66:
		case 0x68:
		case 0x6A:
			return sys_in8(port);
	       
		/*
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
		case 0x20:
		case 0x22:
		case 0x24:
		case 0x26:
			return sys_in8(port);
*/
		default:
			fprintf(stderr, "Unknown IN16: port=%04x\n", port); 
			break;
	}
	return 0xFFFF;
}


void XI_ENV::clock_tick()
{
	PC_ENV::clock_tick();

	m_pit.tick();
	m_sio.tick();
	m_8089.tick();
	m_pic.tick();
//	m_ctc.tick();

}



void XI_ENV::reboot()
{
	m_8089.reset();
	m_pio.reset();
	m_pic.reset();
	m_pit.reset();
	m_fdc.reset();
	m_winch.reset();
	PC_ENV::reboot();
/* Enable all three timer channels */
	m_pit.setGate(0, true);
	m_pit.setGate(1, true);
	m_pit.setGate(2, true);
	m_pit.setListener(0, this);
/* In the PC/XI, the floppy motors are always on */
	m_fdc.set_motor(1);
	m_locked = false;
}


UiEvent XI_ENV::floppy_insert(int drive, std::string filename, const char *type)
{
	m_fdc.insert(drive, filename, type);
}

void XI_ENV::floppy_eject(int drive)
{
	m_fdc.eject(drive);
}


void XI_ENV::receive_key(u_int8 value)
{
	m_sio.write_rx(1, value);
}

void XI_ENV::poll_events()
{
	if (gl_f10count)
	{
		--gl_f10count;
		if (!gl_f10count) m_keyboard->send_time();
	}
	PC_ENV::poll_events();
}


void XI_ENV::onTimerFire(I8253 *source, int channel, bool value)
{
	m_pic.setirq(6, value);
}


u_int8 XI_ENV::fetch(u_int32 offset)
{
	if (offset >= m_memsize->m_memsize && offset < 0xF0000) return 0xFF;

	if (offset < 0xF0000 || offset >= 0xF8000) return m_mem_base[offset];

	return m_mem_base[offset & 0xF0FFF];
}

void XI_ENV::store(u_int32 offset, u_int8 value)
{
	if (offset < m_memsize->m_memsize)
	{
		m_mem_base[offset] = value;
		return;
	}
	if (offset < 0xF8000)  m_mem_base[offset & 0xF0FFF] = value;
}

void XI_ENV::sys_lock()
{
	m_locked = true;
}
