/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2008  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include "qdae.hxx"
#include "device.hxx"
#include "memsize.hxx"

#include "UiControl.hxx"
#include "UiContainer.hxx"
#include "UiLabel.hxx"
#include "UiSeparator.hxx"
#include "UiSetting.hxx"
#include "UiCommand.hxx"
#include "UiMenu.hxx"

Memsize::Memsize() : Device("memory")
{
	m_memsize = 0xE0000;	// 896k
	m_max = 0xE0000;	// Max 896k
}

Memsize::~Memsize()
{

}

void Memsize::reset()
{

}


bool Memsize::parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	int i;
	bool b = parseInteger(doc, ns, node, "size", &i);
	if (b && i <= m_max) m_memsize = i;
	return b;
}


bool Memsize::storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	return storeInteger(doc, ns, node, "size", m_memsize);
}


bool Memsize::hasSettings(SDLKey *key, string &caption)
{
	*key = SDLK_m;
	caption = "  Memory  ";
	return true;
}

UiEvent Memsize::settings(UiDrawer *d)
{
	int sel = -1;
	UiEvent e;

        while (1)
        {
                UiMenu uim(d);
                bool bEnabled;

                bEnabled = isEnabled();
                uim.add(new UiLabel("  Memory  ", d));
                uim.add(new UiSeparator(d));
                uim.add(new UiSetting(SDLK_2, (m_memsize == 256*1024), "  256k  ", d));
                uim.add(new UiSetting(SDLK_5, (m_memsize == 512*1024), "  512k  ", d));
                uim.add(new UiSetting(SDLK_6, (m_memsize == 640*1024), "  640k  ", d));
                uim.add(new UiSetting(SDLK_7, (m_memsize == 768*1024), "  768k  ", d));
                uim.add(new UiSetting(SDLK_8, (m_memsize == 896*1024), "  896k  ", d));
		if (m_max > 0xE0000)
		{
              	  uim.add(new UiSetting(SDLK_9, (m_memsize == 960*1024), "  960k  ", d));
               	  uim.add(new UiSetting(SDLK_x, (m_memsize == 960*1024-16), "  959.9k  ", d));
		}
	       	uim.add(new UiSeparator(d));
	        uim.add(new UiCommand(SDLK_ESCAPE, "  EXIT  ", d));
		uim.setSelected(sel);
		d->centreContainer(uim);	
		e = uim.eventLoop();
                if (e == UIE_QUIT) return e;
                sel = uim.getSelected();
                switch(uim.getKey(sel))
		{
			case SDLK_2:	  m_memsize = 256 * 1024; break;
			case SDLK_5:	  m_memsize = 512 * 1024; break;
			case SDLK_6:	  m_memsize = 640 * 1024; break;
			case SDLK_7:	  m_memsize = 768 * 1024; break;
			case SDLK_8:	  m_memsize = 896 * 1024; break;
			case SDLK_9:	  m_memsize = 960 * 1024; break;
			case SDLK_x:	  m_memsize = 960 * 1024 - 16; break;
			case SDLK_ESCAPE: return UIE_CONTINUE;
			default:	break;
		}
	}
	if (m_memsize > m_max) m_memsize = m_max;
	return UIE_CONTINUE;
}

