/************************************************************************* 
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005,2007  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include "qdae.hxx"
#include "qdae_ui.hxx"
#include "qdae_files.hxx"
#include "sdl_video.hxx"
#include "microscreen.hxx"
#include "SdlContext.hxx"
#include "SdlTerm.hxx"
#include "sdl_keyboard.hxx"
#include "wd2797.hxx"

static int st_lcd_coords[2] =
{
	260, 45
};

static int st_led_coords[12] =
{
	250, 128,
	333, 128,
	417, 128,
	500, 128,
	584, 128,
	667, 128
};

static int st_hotspots[] =
{
	264, 88,
	347, 88,
	430, 88,
	513, 88,
	596, 88,
	680, 88
};

static u_int8 st_uscreen_font[] =
{
#include "uscreen.inc"
};

Microscreen::Microscreen(SDL_Surface *video) : Device("microscreen")
{
	m_video = video;
	m_uscreen = NULL;
	m_led[0] = NULL;
	m_led[1] = NULL;
	m_x = 0;
	m_y = 0;

	string strbitmap = findQdaeFile(FT_BITMAPS, "uscreen.bmp", "rb");
	SDL_Surface *bmp = SDL_LoadBMP(strbitmap.c_str());

	if (!bmp)
	{
		gl_ui->report("Could not open uscreen.bmp", "  Exit  ");
		exit(1);
	}
	m_uscreen = SDL_DisplayFormat(bmp);
	if (!m_uscreen) m_uscreen = bmp;
	else		SDL_FreeSurface(bmp);	

	int n;
	for (n = 0; n < 2; n ++)
	{
		m_led[n] = SDL_CreateRGBSurface(SDL_SWSURFACE, 16, 16,
				video->format->BitsPerPixel,
				video->format->Rmask,
				video->format->Gmask,
				video->format->Bmask,
				video->format->Amask);
	}
	SDL_Rect rcfrom = {16,16, 16, 16 };
	SDL_Rect rcto   = { 0, 0, 16, 16 };

	/* Copy first LED bitmap from (16,16) */
	SDL_BlitSurface(m_uscreen, &rcfrom, m_led[0], &rcto);
	rcfrom.x = 48;	
	/* Copy second LED bitmap from (48,16) */
	SDL_BlitSurface(m_uscreen, &rcfrom, m_led[1], &rcto);
	/* Now copy a grey square over the two 'template' LEDs */
	rcfrom.x = 32;	
	rcto.x = 16;
	rcto.y = rcfrom.y;
	SDL_BlitSurface(m_uscreen, &rcfrom, m_uscreen, &rcto);
	rcto.x = 48;	
	SDL_BlitSurface(m_uscreen, &rcfrom, m_uscreen, &rcto);

	m_bg = SDL_MapRGB(video->format, 0x71, 0xa7, 0xbf);
	m_fg = SDL_MapRGB(video->format, 0x3a, 0x5d, 0x6e);

	reset();
}

Microscreen::~Microscreen()
{
	if (m_uscreen) SDL_FreeSurface(m_uscreen);
	if (m_led[0])  SDL_FreeSurface(m_led[0]);
	if (m_led[1])  SDL_FreeSurface(m_led[1]);
}


void Microscreen::repaint(int x, int y, int w, int h)
{
	SDL_Rect rcf, rct;

	m_x = x;
	m_y = y;
	rcf.x = 0;
	rcf.y = 0;
	rcf.w = w;
	rcf.h = h;
			
	rct.x = x;
	rct.y = y;
	rct.w = w;
	rct.h = h;

	updateText(m_uscreen, 0, 0);
	updateLEDs(m_uscreen, 0, 0);

	SDL_BlitSurface(m_uscreen, &rcf, m_video, &rct);
}

void Microscreen::setLEDs(u_int8 value)
{
	m_ledValue = (value >> 2) & 0x3F;
	updateLEDs(m_uscreen,   0,   0);
	updateLEDs(m_video,   m_x, m_y);
}

void Microscreen::updateChar(SDL_Surface *s, int x, int y, char ch, bool bCursor)
{
	u_int8 *bitmap = st_uscreen_font + 4 + 7*ch;
	int y1, x1, x2, y2;
	unsigned offset, xoffset;
	unsigned char *framebuf = (unsigned char *)(s->pixels);
	u_int8 mask, cur;

	offset = (y * s->pitch) + (x * s->format->BytesPerPixel);
	for (y1 = 0; y1 < 7; y1++)
	{
		for (y2 = 0; y2 < 2; y2++)
		{
			mask = 0x80;
			cur = bitmap[y1];
			/* Cursor is a simple underline */
			if (bCursor && (y1 == 6)) cur = 0xF8;
			/* If uScreen disabled all output is blank */
			if (!m_visible) cur = 0;
			xoffset = offset + (y1 * 2 + y2) * s->pitch;
			for (x1 = 0; x1 < 5; x1++)
			{
				for (x2 = 0; x2 < 2; x2++)
				{
					Uint32 *ptr = (Uint32 *)(framebuf + xoffset);
					*ptr = (cur & mask) ? m_fg : m_bg;
					xoffset += s->format->BytesPerPixel;	
				}
				mask = mask >> 1;
			}
		}	
	}
}

void Microscreen::hideCur()
{
	m_curShown = 0;
	updateText(m_uscreen,   0,   0, m_curx, m_cury);
	updateText(m_video,   m_x, m_y, m_curx, m_cury);
}


void Microscreen::showCur()
{
	m_curShown = 1;
	updateText(m_uscreen,   0,   0, m_curx, m_cury);
	updateText(m_video,   m_x, m_y, m_curx, m_cury);
}

void Microscreen::updateText(SDL_Surface *s, int x, int y, int chx, int chy)
{
	if (chx >= 0 && chy >= 0)
	{
		updateChar(s, st_lcd_coords[0] + x + 12 * chx, 
			      st_lcd_coords[1] + y + 18 * chy, 
				m_data[chy][chx],
				(m_curShown && m_curx == chx && m_cury == chy));
	}
	else for (chy = 0; chy < 2; chy++)
	{
		for (chx = 0; chx < 40; chx++)
		{
			updateChar(s, st_lcd_coords[0] + x + 12 * chx, 
			      	      st_lcd_coords[1] + y + 18 * chy, 
				m_data[chy][chx],
				(m_curShown && m_curx == chx && m_cury == chy));	
		}
	}
}

void Microscreen::updateLEDs(SDL_Surface *s, int x, int y)
{
	int n, mask;

	SDL_Rect rcf = {0, 0, 12, 13}, 
	         rct = {0, 0, 12, 13};

	mask = 1;
	for (n = 0; n < 6; n++)
	{
		rct.x = st_led_coords[2*n  ] + x;
		rct.y = st_led_coords[2*n+1] + y;
		if (m_ledValue & mask)	
			SDL_BlitSurface(m_led[1], &rcf, s, &rct);
		else	SDL_BlitSurface(m_led[0], &rcf, s, &rct);
		mask = mask << 1;
	}
}

void Microscreen::clearData()
{
	// For some reason this caused a messy crash on Win32 QDAE
	//memset(m_data, 0, sizeof(m_data));
	for (int n = 0; n < 2; n++)
		for (int m = 0; m < 40; m++)
			m_data[n][m] = 0;
}

void Microscreen::reset()
{
	m_clickkey = -1;
	m_ledValue = 0;
	m_curx = 0;
	m_cury = 0;
	m_curShown = 0;
	m_visible = true;
	clearData();
}

void Microscreen::addChar(u_int8 ch)
{
	m_data[m_cury][m_curx] = ch;
/* If the cursor is shown, this happens as part of curRight(). If not... */
	if (!m_curShown)
	{
		updateText(m_uscreen,   0,   0, m_curx, m_cury);
		updateText(m_video,   m_x, m_y, m_curx, m_cury);
	}
	curRight();
}


void Microscreen::setCurPos(int x, int y)
{
	int ocx = m_curx;
	int ocy = m_cury;

	m_curx = x;
	m_cury = y;

	if (m_curShown)
	{
		updateText(m_uscreen,   0,   0, ocx, ocy);
		updateText(m_video,   m_x, m_y, ocx, ocy);
		updateText(m_uscreen,   0,   0, m_curx, m_cury);
		updateText(m_video,   m_x, m_y, m_curx, m_cury);
	}
}

void Microscreen::curLeft()
{
	int ocy = m_cury;
	int ocx = m_curx;
	--m_curx;
	if (m_curx < 0)
	{
		m_curx = 39;
		m_cury ^= 1;
	}
	if (m_curShown)
	{
		updateText(m_uscreen,   0,   0, ocx, ocy);
		updateText(m_video,   m_x, m_y, ocx, ocy);
		updateText(m_uscreen,   0,   0, m_curx, m_cury);
		updateText(m_video,   m_x, m_y, m_curx, m_cury);
	}
}


void Microscreen::curRight()
{
	int ocy = m_cury;
	int ocx = m_curx;
	++m_curx;
	if (m_curx >= 40)
	{
		m_curx = 0;
		m_cury ^= 1;
	}
	if (m_curShown)
	{
		updateText(m_uscreen,   0,   0, ocx, ocy);
		updateText(m_video,   m_x, m_y, ocx, ocy);
		updateText(m_uscreen,   0,   0, m_curx, m_cury);
		updateText(m_video,   m_x, m_y, m_curx, m_cury);
	}
}

void Microscreen::setVisible(bool b)
{
	if (m_visible == b) return;

	m_visible = b;	
	updateText(m_uscreen,   0,   0);
	updateText(m_video,   m_x, m_y);
}

void Microscreen::clear()
{
	clearData();
	updateText(m_uscreen,   0,   0);
	updateText(m_video,   m_x, m_y);
}

void Microscreen::updateTime()
{
	time_t t;
	struct tm *ptm;
	char tmbuf[40];
	bool updated = false;
	int n;	

	if (!m_showTime) return;
	time(&t);
	ptm = localtime(&t);
	strftime(tmbuf, sizeof(tmbuf), "%a %d %b %y %H:%M:%S", ptm);
	for (n = 0; n < 22; n++)
	{
		if (m_data[0][n + 18] != tmbuf[n])
		{
			m_data[0][n + 18] = tmbuf[n];
			updated = true;
			updateText(m_uscreen, 0, 0,     n + 18, 0);
			updateText(m_video,   m_x, m_y, n + 18, 0);
		}
	}
}

UiEvent Microscreen::handleEvent(SDL_Event *e, int *k)
{
	int n, hy, hx;

	switch(e->type)
	{
		case SDL_QUIT: return UIE_QUIT;
		case SDL_MOUSEBUTTONUP:
			if (m_clickkey != -1)
			{
				n = m_clickkey;
				m_clickkey = -1;
				*k = n | 0x80;	
				return UIE_OK;
			}
			break;
		case SDL_MOUSEMOTION:
			if (e->motion.y >= m_y &&
			    e->motion.y <  (m_y + m_uscreen->h))
			{
				return UIE_OK;
			}
			break;
		case SDL_MOUSEBUTTONDOWN:
			if (e->button.y < m_y ||
			    e->button.y >= m_y + m_uscreen->h) 
				return UIE_CONTINUE;
			for (n = 0; n < 6; n++)
			{
				hx = st_hotspots[n * 2];
				hy = st_hotspots[n * 2 + 1];
				if (e->button.x >= m_x + hx      &&
				    e->button.x <  m_x + hx + 58 &&	
				    e->button.y >= m_y + hy      &&
				    e->button.y <  m_y + hy + 54)
				{
					m_clickkey = n + 9;
					*k = m_clickkey;
					return UIE_OK;
				}
			}
			return UIE_CANCEL;
				
	}
	return UIE_CONTINUE;
}


void Microscreen::setTime(bool b)
{
	m_showTime = b;
}


bool Microscreen::parseNode(xmlDoc*, xmlNs*, xmlNode*)
{
	return true;
}


bool Microscreen::storeNode(xmlDoc*, xmlNs*, xmlNode*)
{
	return true;
}

