/************************************************************************* 
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/


#include "qdae.hxx"
#include "qdae_ui.hxx"
#include "sdl_video.hxx"
#include "SdlContext.hxx"
#include "SdlTerm.hxx"
#include "sdl_keyboard.hxx"
#include "wd2797.hxx"

#define SCREEN_DEPTH 32
#define SCREEN_FLAGS 0
#define FPS_DELAY 100	/* Giving 10fps */


Video::Video(PC_ENV *m, std::string name, int w, int h) :
	Device("video", name)
{
	width = w;
	height = h;
	mouse_captured = false;
	host = m;
	term = NULL;
}

Video::~Video()
{
	if (mouse_captured)
	{
		mouse_captured = 0;
		SDL_WM_GrabInput(SDL_GRAB_OFF);
		SDL_ShowCursor(SDL_ENABLE); 
	}
}


int video_painter(void *param)
{
	Video *v = (Video *)param;

	return v->paint();
}


void Video::init()
{
	int n;
	SDL_Rect rcBounds;
	Uint32 initflags = SDL_INIT_VIDEO;

	if (SDL_Init(initflags) < 0)
	{
		fprintf(stderr, "Couldn't initialise SDL: %s\n",
				SDL_GetError());
		exit(1);
	}
	vidSurface = SDL_SetVideoMode(width,
			height,
			SCREEN_DEPTH,
			SCREEN_FLAGS);
	if (!vidSurface)
	{
		fprintf(stderr, "Can't open surface\n");
		SDL_Quit();
		exit(1);
	}
	SDL_WM_SetCaption("QDAE", "QDAE"); 
	gl_ui = new QDAE_UI(host, vidSurface);

	rcBounds.x = 0;
	rcBounds.y = vidSurface->h - extra_height;
	rcBounds.w = width;
	rcBounds.h = extra_height;
	term = new SdlTerm(vidSurface, &rcBounds);
	term->setBackground(0x40, 0x40, 0x40);
	term->clearWindow();
	gl_ledsdirty = 1;
	SDL_CreateThread(video_painter, this);
}


/* Thanks to Jerome Vernet for mouse capture */
void Video::captureMouse(bool captured)
{
	if (captured)
	{
		SDL_WM_GrabInput(SDL_GRAB_ON);
		SDL_ShowCursor(SDL_DISABLE);
		mouse_captured = 1; 
	}
	else
	{
		mouse_captured = 0;
		SDL_WM_GrabInput(SDL_GRAB_OFF);
		SDL_ShowCursor(SDL_ENABLE); 
	}
}


bool Video::isMouseCaptured()
{
	return mouse_captured;
}

void Video::reset()
{

}


bool Video::storeNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
	return true;
}

bool Video::parseNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
	return true;
}

u_int8 Video::read(u_int16 port)
{
	return 0xFF;
}

