/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2008  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include "qdae.hxx"
#include "device.hxx"
#include "speed.hxx"

#include "UiControl.hxx"
#include "UiContainer.hxx"
#include "UiLabel.hxx"
#include "UiSeparator.hxx"
#include "UiSetting.hxx"
#include "UiCommand.hxx"
#include "UiMenu.hxx"

Speed::Speed() : Device("speed")
{
	m_speed = 2500;
}

Speed::~Speed()
{

}

void Speed::reset()
{

}


bool Speed::parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	int i;
	bool b = parseInteger(doc, ns, node, "cycles", &i);
	if (b) m_speed = i;
	return b;
}


bool Speed::storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	return storeInteger(doc, ns, node, "cycles", m_speed);
}


bool Speed::hasSettings(SDLKey *key, string &caption)
{
	*key = SDLK_s;
	caption = "  Speed  ";
	return true;
}

UiEvent Speed::settings(UiDrawer *d)
{
	int sel = -1;
	UiEvent e;

        while (1)
        {
                UiMenu uim(d);
                bool bEnabled;

                bEnabled = isEnabled();
                uim.add(new UiLabel("  Speed  ", d));
                uim.add(new UiSeparator(d));
                uim.add(new UiSetting(SDLK_1, (m_speed == 2500), "  Normal  ", d));
                uim.add(new UiSetting(SDLK_2, (m_speed == 5000), "  Double  ", d));
                uim.add(new UiSetting(SDLK_3, (m_speed == 7500), "  Triple  ", d));
                uim.add(new UiSetting(SDLK_4, (m_speed == 10000), "  Quadruple  ", d));
                uim.add(new UiSetting(SDLK_u, (m_speed == 0), "  Unlimited  ", d));
	       	uim.add(new UiSeparator(d));
	        uim.add(new UiCommand(SDLK_ESCAPE, "  EXIT  ", d));
		uim.setSelected(sel);
		d->centreContainer(uim);	
		e = uim.eventLoop();
                if (e == UIE_QUIT) return e;
                sel = uim.getSelected();
                switch(uim.getKey(sel))
		{
			case SDLK_1:	  m_speed = 2500; break;
			case SDLK_2:	  m_speed = 5000; break;
			case SDLK_3:	  m_speed = 7500; break;
			case SDLK_4:	  m_speed = 10000; break;
			case SDLK_u:	  m_speed = 0; break;
			case SDLK_ESCAPE: return UIE_CONTINUE;
			default:	break;
		}
	}
	return UIE_CONTINUE;
}

