
/* 
 * Dos/PC Emulator
 * Copyright (C) 1991 Jim Hudgens
 * QDAE Copyright 2005, 2007, 2008 John Elliott
 * 
 * 
 * The file is part of GDE.
 * 
 * GDE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 * 
 * GDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GDE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 *
 */

/* Although this is based on the GDE system configuration parser, very 
 * little of the original code is left. It's been rewritten to use the same
 * JOYCE/QDAE XML-format configuration code as everything else. */

#include "qdae.hxx"
#include "qdae_files.hxx"
#include "UiControl.hxx"
#include "UiContainer.hxx"
#include "UiLabel.hxx"
#include "UiSeparator.hxx"
#include "UiCommand.hxx"
#include "UiSetting.hxx" 
#include "UiMenu.hxx"
#include "UiTextInput.hxx"
#include "UiScrollingMenu.hxx"

#define XMLNS  "http://www.seasip.demon.co.uk/qdae_general.dtd"

extern string *gl_homeRoot;	// Emulator data directory
extern string *gl_keyboard;	// Keyboard layout

GeneralSettings::GeneralSettings() : Device("general")
{
	defaults();
	m_settings = xmlNewDoc((xmlChar *)"1.0");
        if (m_settings)      
        {                    
                xmlNsPtr ns; 
                xmlNodePtr root;

                root = xmlNewNode(NULL, (xmlChar *)"settings");
                xmlDocSetRootElement(m_settings, root);

                ns = xmlNewNs(root, (xmlChar *)XMLNS, NULL);
        }
}

GeneralSettings::~GeneralSettings()
{
}

void GeneralSettings::reset()
{
}

bool GeneralSettings::hasSettings(SDLKey *key, string &caption)
{
	*key = SDLK_g;
	caption = "  General settings  ";
	return true;
}

bool GeneralSettings::parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	string filename;
	bool b;

	b = parseFilename(doc, ns, node, "homedir", filename);
	if (b)
	{	
		*gl_homeRoot = filename;
	}
	b = parseFilename(doc, ns, node, "keyboard", filename);
	if (b)
	{	
		*gl_keyboard = filename;
	}
	b = parseFilename(doc, ns, node, "debug", filename);
	if (b)
	{
		parseDebug(filename.c_str());
	}
	b = parseFilename(doc, ns, node, "systype", filename);
	if (b)
	{
		type = machine_type(filename.c_str());
	}
}

static const char *st_systypes[] =
{
	"default",
	"xi",
	"fp",
	"f1",
	"xen"
};

bool GeneralSettings::storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	storeFilename(doc, ns, node, "homedir", *gl_homeRoot);
	storeFilename(doc, ns, node, "keyboard", *gl_keyboard);
	storeFilename(doc, ns, node, "debug", storeDebug());
	if (type != DEFAULT)
	{
		storeFilename(doc, ns, node, "systype", st_systypes[type]);
	}
}



#define SAMESTRING(x,y)  (strncmp(x,y,strlen(y)) == 0)

string GeneralSettings::storeDebug()
{
	string s = "";

	if (debug & DEBUG_SVC_F)    s += ",DBGSVC";
	if (debug & DEBUG_DECODE_F) s += ",DBGDECODE";
	if (debug & DEBUG_STEP_F)   s += ",DBGSTEP";
	if (debug & DEBUG_FS_F)   s += ",DBGFS";
	if (debug & DEBUG_PROC_F)   s += ",DBGPROC";
	if (debug & DEBUG_SYSINT_F)   s += ",DBGSYSINT";
	if (debug & DEBUG_TRACECALL_F)   s += ",DBGTRACECALL";
	if (debug & DEBUG_TRACE_F)   s += ",DBGTRACE";
	if (debug & DEBUG_DISASSEMBLE_F)   s += ",DBGDISASS";

	if (s == "") return s;
	return s.substr(1);	// Lose the initial comma
}

void GeneralSettings::parseDebug(const char *p)
{
	while(p != NULL)
	{
		if (SAMESTRING(p,"DBGSVC"))            debug |= DEBUG_SVC_F;
		else if (SAMESTRING(p,"DBGDECODE"))    debug |= DEBUG_DECODE_F;
		else if (SAMESTRING(p,"DBGSTEP"))      debug |= DEBUG_STEP_F;
		else if (SAMESTRING(p,"DBGFS"))        debug |= DEBUG_FS_F;
		else if (SAMESTRING(p,"DBGPROC"))      debug |= DEBUG_PROC_F;
		else if (SAMESTRING(p,"DBGSYSINT"))    debug |= DEBUG_SYSINT_F;
		else if (SAMESTRING(p,"DBGTRACECALL")) debug |= DEBUG_TRACECALL_F;
		else if (SAMESTRING(p,"DBGTRACE"))     debug |= DEBUG_TRACE_F;
		else if (SAMESTRING(p,"DBGDISASS"))    debug |= DEBUG_DISASSEMBLE_F;
		p = strchr (p,',');
		if (p) ++p;
	}
}



void GeneralSettings::defaults()
{
	debug = 0;
	type = DEFAULT;
	if (*gl_homeRoot == "")
	{
		*gl_homeRoot = DEF_HOMEDIR;
	}
	if (*gl_keyboard == "")
	{
		*gl_keyboard = DEF_KEYBOARD;
	}
}

bool GeneralSettings::saveConfig()
{
        xmlNodePtr root = xmlDocGetRootElement(m_settings);
        xmlNsPtr   ns   = xmlSearchNsByHref(m_settings, root, (xmlChar *)XMLNS);

	storeNode(m_settings, ns, root);

	if (m_filename == "")
	{
		m_filename = getHomeDir();
		m_filename += "/" RCFILE;
	}

	if (xmlSaveFile(m_filename.c_str(), m_settings) < 0) return false;
	return true;
}

bool GeneralSettings::loadConfig(const std::string filename)
{
        xmlDocPtr doc;
        xmlNsPtr ns;
        xmlNodePtr cur;

	m_filename = filename;
//
// Do a lot of sanity checks on the hardware XML.
//
        doc = xmlParseFile(filename.c_str());
        if (doc == NULL)	// Can't open it
        {
                fprintf(stderr, "%s: File does not exist or is not in XML format\n", filename.c_str());
                return false;
        }
        cur = xmlDocGetRootElement(doc);
        if (cur == NULL)	// No root element
        {
                fprintf(stderr, "%s: File is empty or not in XML format\n", filename.c_str());
                xmlFreeDoc(doc);
                return false;
        }
        ns = xmlSearchNsByHref(doc, cur, (xmlChar *)XMLNS);
        if (ns == NULL)		// No namespace
        {
                fprintf(stderr, "%s: No QDAE namespace found\n", filename.c_str());
                xmlFreeDoc(doc);
                return false;
        }
        if (strcmp((char *)cur->name, "settings")) // Wrong root
        {
                fprintf(stderr, "%s: root node is not of type 'settings'.", filename.c_str());
                xmlFreeDoc(doc);
                return false;
        }
	// Newly-loaded DOM is okay.
	if (m_settings) xmlFreeDoc(m_settings);
	m_settings = doc; 
	return parseNode(doc, ns, cur);
	
}



inline void nextarg(int *n, int argc, char **argv, const char *s)
{
	++n[0];
	if (n[0] >= argc)
	{
		fprintf(stderr, "%s: %s option must be followed by a %s.\n", argv[0], argv[n[0] - 1], s);	
		exit(1);
	}
}	


void GeneralSettings::configArgs(int argc, char **argv)
{
	int n;
	for (n = 1; n < argc; n++)
	{
		if (!strcmp(argv[n], "--"))	/* end of options */
			return;
		if (!strcmp(argv[n], "-k") || !strcmp(argv[n], "-K"))
		{
			nextarg(&n, argc, argv, "keyboard layout name");
			*gl_keyboard = argv[n];
			continue;
		}
		if (!strcmp(argv[n], "-h") || !strcmp(argv[n], "-H") &&
		     n < (argc - 1))
		{
			nextarg(&n, argc, argv, "home directory name");
			*gl_homeRoot = argv[n];
			continue;
		}
		if (!strcmp(argv[n], "-h") || !strcmp(argv[n], "-H") &&
		    !strcmp(argv[n], "-?") || !strcmp(argv[n], "--help"))
		{
			syntax(argv);
			continue;
		}
		fprintf(stderr, "%s:Unrecognised option '%s'\n\n",
				argv[0], argv[n]);
		syntax(argv);
	}
}



UiEvent GeneralSettings::settings(UiDrawer *d)
{
	int sel = -1;
	UiEvent uie;

	while (1)
	{
		UiMenu uim(d);
		uim.add(new UiLabel("  General settings  ", d));
		uim.add(new UiSeparator(d));
#ifdef WIN32
		uim.add(new UiTextInput(SDLK_h, "  Home directory: ________________________________  ",d));
		uim.add(new UiTextInput(SDLK_d, "  Data subdirectory: ________________________________  ",d));
#else
		uim.add(new UiTextInput(SDLK_d, "  Data directory: ________________________________  ",d));
#endif
		
		uim.add(new UiTextInput(SDLK_k, "  Keyboard layout: ________  ",d));
		uim.add(new UiSeparator(d));
		uim.add(new UiLabel("  Default system type:  ", d));
		uim.add(new UiSetting(SDLK_x, type == PCSERIES, "  Xi / PC series  ",  d));	
//		uim.add(new UiSetting(SDLK_p, type == RASCALA,  "  Portable  ",  d));	
		uim.add(new UiSetting(SDLK_f, type == RASCALC,  "  F-series  ",  d));	
//		uim.add(new UiSetting(SDLK_e, type == XEN,      "  Xen  ",  d));	
		uim.add(new UiSeparator(d));
                uim.add(new UiCommand(SDLK_ESCAPE, "  EXIT  ", d));
                d->centreContainer(uim);
                uim.setSelected(sel);
#ifdef WIN32
                ((UiTextInput &)uim[2]).setText(getHomeDir());
                ((UiTextInput &)uim[3]).setText(*gl_homeRoot);
                ((UiTextInput &)uim[4]).setText(*gl_keyboard);
#else
                ((UiTextInput &)uim[2]).setText(*gl_homeRoot);
                ((UiTextInput &)uim[3]).setText(*gl_keyboard);
#endif
		uie = uim.eventLoop();
		if (uie == UIE_QUIT) return uie;
		sel = uim.getSelected();
		switch(uim.getKey(sel))
		{
			case SDLK_x: type = PCSERIES; break;
			case SDLK_f: type = RASCALC; break;
			case SDLK_p: type = RASCALA; break;
			case SDLK_e: type = XEN; break;
#ifdef _WIN32
			case SDLK_h: setHomeDir(((UiTextInput &)uim[2]).getText()); break;
			case SDLK_d: *gl_homeRoot = ((UiTextInput &)uim[3]).getText(); break;
			case SDLK_k: *gl_keyboard = ((UiTextInput &)uim[4]).getText(); break;
#else
			case SDLK_d: *gl_homeRoot = ((UiTextInput &)uim[2]).getText(); break;
			case SDLK_k: *gl_keyboard = ((UiTextInput &)uim[3]).getText(); break;
#endif
			case SDLK_ESCAPE:
				     return UIE_CONTINUE;
		}
	}
	return UIE_CONTINUE;
}
