/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005-6  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#ifndef WD2797_HXX_INCLUDED
#define WD2797_HXX_INCLUDED 1

#define MAX_FLOPPIES 2


EXT const char *floppy_type [MAX_FLOPPIES];
EXT string *floppy_filename [MAX_FLOPPIES];
EXT bool   floppy_readonly  [MAX_FLOPPIES];
EXT bool   floppy_indicator [MAX_FLOPPIES];


class WD2797 : public Device
{
public:
	WD2797(PC_ENV *m);
	virtual ~WD2797();

	virtual void reset();
	void stop();

	void write(u_int16 port, u_int8 value);
	u_int8 read(u_int16 port);

	void eject(int drive);
	UiEvent insert(int drive, string filename, const char *type);

	void select_drive(int drive);
	void load_head(bool loaded);
	void set_motor(u_int8 on);

	virtual bool parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);
	virtual bool storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);

private:
	void xlt_dsk_error(dsk_err_t err);
	void read_sector(u_int8 cmd_byte);
	void write_sector(u_int8 cmd_byte);
	void write_sector_end();
	void probe_devices();
	void begin_command(u_int8 value);
	void write_command(u_int8 value);

	PC_ENV *pc_env;
	u_int8 fdc_reg[4];

	int cur_cyl[MAX_FLOPPIES];
	int busy;
	int drive;
	int motor;
	int irq_flags;
	u_int8 transfer_buf[8192];
	u_int8 write_cmd;
	int transfer_count;
	int transfer_ptr;

	DSK_PDRIVER dsk_drive[MAX_FLOPPIES];
	DSK_GEOMETRY dsk_geom[MAX_FLOPPIES];
};

#endif //ndef WD2797_HXX_INCLUDED

