/************************************************************************* 
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005-6  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/


#include "qdae.hxx"
#include "qdae_ui.hxx"
#include "qdae_files.hxx"
#include "machine_xi.hxx"
#include "sdl_keyboard.hxx"
#include "xi_keyboard.hxx"
#include "microscreen.hxx"

XI_Keyboard::XI_Keyboard(XI_ENV *m) : Keyboard((PC_ENV *)m, "xi")
{

}

XI_Keyboard::~XI_Keyboard()
{

}

/* Generate the "Here is the time" packet which the Rascal keyboard can be 
 * made to transmit */
void XI_Keyboard::send_time()
{
	u_int8 pkt[13];
	time_t t;
	struct tm *ptm;
	int n;

	time (&t);
	ptm = localtime(&t);
	pkt[ 0] = 0xF0 | (((ptm->tm_year % 100) / 10));
	pkt[ 1] = 0xF0 | ((ptm->tm_year % 10));
	pkt[ 2] = 0xF0 | ((ptm->tm_mon / 10));
	pkt[ 3] = 0xF0 | ((ptm->tm_mon % 10));
	pkt[ 4] = 0xF0 | ((ptm->tm_mday / 10));
	pkt[ 5] = 0xF0 | ((ptm->tm_mday % 10));
	pkt[ 6] = 0xF0 | ((ptm->tm_wday % 10));
	pkt[ 7] = 0xF0 | ((ptm->tm_hour / 10));
	pkt[ 8] = 0xF0 | ((ptm->tm_hour % 10));
	pkt[ 9] = 0xF0 | ((ptm->tm_min / 10));
	pkt[10] = 0xF0 | ((ptm->tm_min % 10));
	pkt[11] = 0xF0 | ((ptm->tm_sec / 10));
	pkt[12] = 0xF0 | ((ptm->tm_sec % 10));
	for (n = 0; n < 13; n++)
	{
		((XI_ENV *)host)->receive_key(pkt[n]);
	}
//	Sys.debug = 3;
}


//
// Note for posterity: If you come here trying to track down a problem with
// the mouse: The "PC / XI Keyboard Mouse" driver version 1.0.5 is buggy so
// try with a later version such as 1.2.2. I'm sure I've instrumented this
// piece of code at least twice trying to work out what's wrong!
//
void XI_Keyboard::send_mouse(int btns, int x, int y)
{
	u_int8 pkt[6];

	if (!m_mouseEnabled)
		return;

	pkt[0] = 0xEF;				// Mouse header
	pkt[1] = 0x70 | (btns & 3);		// Mouse buttons
	pkt[2] = 0x70 | ((y >> 4) & 0x0F); 	
	pkt[3] = 0x70 |  (y & 0x0F); 		// Y-coord
	pkt[4] = 0x70 | ((x >> 4) & 0x0F); 	
	pkt[5] = 0x70 |  (x & 0x0F); 		// X-coord

//	fprintf(stderr, "send_mouse: btns=%d x=%d y=%d ", btns, x, y);
	for (int n = 0; n < 6; n++)
	{
//		fprintf(stderr, "%02x ", pkt[n]);
		((XI_ENV *)host)->receive_key(pkt[n]);
	}
//	fprintf(stderr, "\n");
}

inline signed char limit(signed x)
{
	if (x < -127) return -127;
	if (x > 127)  return 127;
	return x;
}

void XI_Keyboard::encode_mouse(int btns, signed dx, signed dy)
{
	signed char deltax, deltay;
	int any = 0;

//	fprintf(stderr, "encode mouse: btns=%d dx=%d dy=%d\n",
//			btns, dx, dy);
	while (dx || dy)
	{
		any = 1;
		deltax = limit(dx);
		deltay = limit(dy);
		send_mouse(btns, deltax, deltay);

		dx -= deltax;
		dy -= deltay;
	}	
	// If it was just a button change, then no movement packet has been 
	// sent.
	//fprintf(stderr, "encode mouse: any=%d\n", any);
	if (!any)
	{
		send_mouse(btns, 0, 0);
	}
}

UiEvent XI_Keyboard::keyboard_event(SDL_Event *e)
{
	int n;
	unsigned shift = 0;
	unsigned ctrl  = 0;
	unsigned repeat= 0;
	static unsigned btn_state = 0;
	int pcxikey;
	UiEvent ev;

	if (m_uscreen)
	{
		ev = m_uscreen->handleEvent(e, &pcxikey);
		switch(ev)
		{
			case UIE_OK:		/* uScreen key */
				((XI_ENV *)host)->receive_key(pcxikey);
				return UIE_CONTINUE;
			case UIE_QUIT:		/* Window closed */
				return UIE_QUIT;
			case UIE_CANCEL:	/* uScreen, but not key */
				return UIE_CONTINUE;
		}	
	
	}	

	switch(e->type)
	{
		case SDL_QUIT: return UIE_QUIT;

		case SDL_KEYDOWN:
			if (e->key.keysym.mod & KMOD_SHIFT) shift = 0x100;
			else				    shift = 0;
			if (e->key.keysym.mod & KMOD_CTRL)  ctrl  = 0x200;
			else				    ctrl  = 0;

/* Convert RightShift + F1-F6 to the MicroScreen keys. */
			if (e->key.keysym.mod & KMOD_RSHIFT &&
			    e->key.keysym.sym >= SDLK_F1 &&
			    e->key.keysym.sym <= SDLK_F6)
			{
/* Only if LeftShift is also held down do we get shifted */
				if (e->key.keysym.mod & KMOD_LSHIFT) 
					shift = 0x100;
				else	shift = 0x000;
				int *j = (int *)&e->key.keysym.sym;
				(*j) += (SDLK_F13 - SDLK_F1);
			}
			if (e->key.keysym.sym >= SDLK_FIRST && 
			    e->key.keysym.sym <= SDLK_LAST)
			{
				if ((e->key.keysym.mod & KMOD_RALT) ||
				    (e->key.keysym.mod & KMOD_MODE))
				  pcxikey = pcxi_key_ralt[e->key.keysym.sym];
				else 
				  pcxikey = pcxi_key_map[e->key.keysym.sym];
				if (pcxikey == -1) break;
				if (pcxikey == -2)
				{
					return gl_ui->mainMenu();		
				}
				pcxikey &= 0x7F;
				((XI_ENV *)host)->receive_key(pcxikey);
			}
			break;
		case SDL_KEYUP:
			if (e->key.keysym.mod & KMOD_SHIFT) shift = 0x100;
			else				    shift = 0;
			if (e->key.keysym.mod & KMOD_CTRL)  ctrl  = 0x200;
			else				    ctrl  = 0;

/* Convert RightShift + XI-F6 to the MicroScreen keys. */
			if (e->key.keysym.mod & KMOD_RSHIFT &&
			    e->key.keysym.sym >= SDLK_F1 &&
			    e->key.keysym.sym <= SDLK_F6)
			{
/* Only if LeftShift is also held down do we get shifted */
				if (e->key.keysym.mod & KMOD_LSHIFT) 
					shift = 0x100;
				else	shift = 0x000;
				int *j = (int *)&e->key.keysym.sym;
				(*j) += (SDLK_F13 - SDLK_F1);
			}

			if (e->key.keysym.sym >= SDLK_FIRST && 
			    e->key.keysym.sym <= SDLK_LAST)
			{
				if ((e->key.keysym.mod & KMOD_RALT) ||
				    (e->key.keysym.mod & KMOD_MODE))
				  pcxikey = pcxi_key_ralt[e->key.keysym.sym];
				else 
				  pcxikey = pcxi_key_map[e->key.keysym.sym];
				if (pcxikey < 0) break;
				pcxikey &= 0x7F;
				((XI_ENV *)host)->receive_key(pcxikey | 0x80);
			}
			break;

		case SDL_MOUSEMOTION:
			m_mousex += e->motion.xrel;
			m_mousey += e->motion.yrel;
			encode_mouse(btn_state, e->motion.xrel, e->motion.yrel);
			break;
		case SDL_MOUSEBUTTONDOWN:
			if (e->button.button == 1) btn_state |= 1;
			else			   btn_state |= 2;
			encode_mouse(btn_state, 0, 0);
			break;
		case SDL_MOUSEBUTTONUP:
			if (e->button.button == 1) btn_state &= ~1;
			else			   btn_state &= ~2;
			encode_mouse(btn_state, 0, 0);
			break;
	}
	return Keyboard::keyboard_event(e);
}


void XI_Keyboard::reset()
{
	m_state = 0;
	m_mouseEnabled = false;
	m_mousex = 0;
	m_mousey = 0;
	((XI_ENV *)host)->receive_key(0xFB);	// Reset finished, KB ready	
}

void XI_Keyboard::receive(SerialChannel *ch)
{
	u_int8 value = ch->read_tx();

	switch(m_state)
	{
		case 0:    break;

		case 0xE3: m_uscreen->setLEDs(value);
			   m_state = 0;
			   return;
	
		default:	   
			fprintf(stderr, "Keyboard: In unknown state %02x\n", 
					m_state);
			m_state = 0;
			break;
	}
	if (value < 0x80)	/* 00-7F are MicroScreen characters */
	{
		m_uscreen->addChar(value);
		return;
	}
	else if (value < 0xD0)
	{
		m_uscreen->setCurPos( (value - 0x80) % 40, 
				      (value - 0x80) / 40);
	}
	else switch(value)
	{
/* D0-D7 control uScreen */
		case 0xD0: m_uscreen->setTime(false);
			   m_uscreen->clear();
			   break;
		case 0xD1: m_uscreen->curLeft(); break;
		case 0xD2: m_uscreen->curRight(); break;
		case 0xD3: m_uscreen->showCur(); break;
		case 0xD4: m_uscreen->hideCur(); break;
		case 0xD5: m_uscreen->setVisible(true); break;
		case 0xD6: m_uscreen->setVisible(false); break;
		case 0xE1: send_time(); break;
		case 0xE2: m_uscreen->setTime(true);
			   break;
		case 0xE3: m_state = 0xE3; break;	// Set uScreen LEDs
		case 0xE5: m_mouseEnabled = true; break;
		case 0xE6: m_mouseEnabled = false; break;
		case 0xE8: reset(); break;
		default:
			fprintf(stderr, "Keyboard: Received byte %02x; discarding\n", value);
	}
}
