/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2007  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include "qdae.hxx"
#include "device.hxx"
#include "machine_xi.hxx"
#include "sdl_video.hxx"
#include "xi_video.hxx"

#define PIO_LOG(x) printf x ; fflush(stdout);

/* This is not a full emulation of an Intel 8255. It emulates the specific
 * way one is used in an Apricot PC/Xi. */

XI_PIO::XI_PIO(XI_ENV *host)
{
	m_host = host;
	reset();
}

XI_PIO::~XI_PIO()
{

}

void XI_PIO::sys_control(u_int8 value)
{
	m_sysctl = value;

	// Bit 0: Reset the CRTC
	if ((value & 1) == 0) 
	{
		m_host->m_video->reset();
	}
	// Bit 3: Display enable. Only do this if the CRTC is not in the 
	// middle of a reset.
	if (value & 1)
	{
		bool enable = ( (value & 8) == 0);
		if (enable != m_host->m_video->isEnabled())
		{
			PIO_LOG(("Video %sable @ %04x:%04x\n",
					enable ? "en" : "dis",
					m_host->R_CS, m_host->R_IP));
		}
		m_host->m_video->enable( (value & 8) == 0);
	}
	else	// If reset signal is low, video is disabled.
	{
		m_host->m_video->enable( false );
	}
	// Bit 4: Text / graphics mode
	((XI_Video *)m_host->m_video)->setTextMode( (value & 16) != 0);
	// Bits 5-6: Drive select
	m_host->m_fdc.select_drive ( (value >> 6) & 1);
	// Bit 2: Head load. Do this one _after_ selecting the correct 
	// drive!
	m_host->m_fdc.load_head( (value & 4) != 0);
	// Bit 7: Parallel port direction
	m_parallel_input = (value & 0x80) != 0;
}

void XI_PIO::write(u_int16 port, u_int8 value)
{
	switch(port)
	{
		case 0:	// Printer data
			// XXX Implement printer support!
			break;
		case 1: // System control
			sys_control(value);
			break;
		case 2: // Printer control
			// XXX Implement printer support!
			break;
		case 3: // PIO control
			// We ignore this and hardcode how we want it to work
			break;
	}
}

u_int8 XI_PIO::read(u_int16 port)
{
	switch(port)
	{
		case 1:	// Last value written 
			return m_sysctl;
		case 2:	// Printer & system status
			// XXX Implement!
			return 0;
	}
	return 0xFF;
}

void XI_PIO::reset()
{
	m_sysctl = 0;
	m_parallel_input = false;
}

bool XI_PIO::parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	return true;
}


bool XI_PIO::storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node)
{
	return true;
}
